/*
    dp : An implementation of the davis puttnam procedure for solving
         propositional satisfiability problems.
    Copyright (C) 1993  Peter Barth

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ------------------------------------------------------------------
  datadefs.h
* -------------------------------------------------------------------
  Global Data structure definition and other global stuff
* ---------------------------------------------------------------- */

#ifndef __datadefs_h
#define __datadefs_h

#include <stdio.h>
#include <stdlib.h>
#include "pblist.h"
#include "silist.h"
#include "sys/times.h"
#include "time.h"

/* To avoid warnings */
#include "protos.h"


#define max(A,B) ((A)>(B) ? (A) : (B))
#define min(A,B) ((A)<(B) ? (A) : (B))

#ifdef VERBOSE
#define DO_VERBOSE(V,T) if (verbosity >= V) {T}
#else
#define DO_VERBOSE(V,T)
#endif

/******************************************************/
/* The datastructures                                 */
/******************************************************/

/* The data how it is read in */
/* a matrix of integers
 *             (Ni Cij)
 * each Cij represents the number of the propositional variable
 * if Cij is negativ then it is the negated occurence
 * Ni represents the number of Literals
 * stored as list of list of ints
 */

extern list problem;


/* The internal datastructure */

/* a) static datastructure 
      The static datastructure is a index structure which allows to
      find all variables in a clause if the clause_id is given (the obvious one)
      and to find all clauses (clause_id's) containing a specific literal
 */
typedef struct vref {
  int variable;
  list clauses;
} tvref;
extern list variable_references;
extern int** lit_to_clauses; /* The variable references as array for faster acces */
extern int** clause_to_lits; /* the clause_id references as array for faster acces */

/* b) dynamic datastructure 
      The dynamic datastructure consists of two single linked lists containing
      i)  all "active variables", that is variables where a value is not yet assigned
      ii) all "active clauses", that is clause_id's which are not yet satisfied
          and the number of active variables in them (length).
      The information fits in a single integer. This implies that the number of
      clauses and the clause length is limited to 65536.
 */
#define GETtaclauselen(i) ((i) & 0x0000FFFF) 
#define GETtaclauseid(i)  ((i) >> 16) 
#define SETtaclauselen(i,v) ((i) = ((i) & 0xFFFF0000) | v) 
#define SETtaclauseid(i,v)  ((i) = ((i) & 0x0000FFFF) | ((v) << 16)) 

extern list active_clauses;     /* int list */
extern list active_variables;   /* int list */
extern silist av,ac;

extern int* ucvs;   /* a stack of unit clause variables */
extern int* ucvsptr;
extern int* ucvscmpptr;

/* For constructing the solution
 */
extern list solution;
extern list solution_all;

/* Some global Flags and statistic information */
extern int showsolution;
extern int verbosity;
extern int timing;
extern int heuristic;
extern int no_nodes;
extern int no_ures;


/* the functions */

void error(char *s);
silist list2silist(list l);
void init_time();
int get_time();

int intcmp(void *d1,void *d2);

void init_datastructures();
void clean_datastructures();


#endif
