/*
    dp : An implementation of the davis puttnam procedure for solving
         propositional satisfiability problems.
    Copyright (C) 1993  Peter Barth

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ------------------------------------------------------------------
   SILIST.C
* -------------------------------------------------------------------
* Auth: pb
* Purp: single linked lists with integers as keys (Sedgewick)
* Hist:==============================================================
* Date         Name    What has been done ?!
* -------------------------------------------------------------------
* -> SILIST.H
* ---------------------------------------------------------------- */

#include "silist.h"
#include "stdlib.h"

silist init_silist()
{
  silist head = (silist)  malloc(sizeof(struct _sinode));
  silist z = (silist) malloc(sizeof(struct _sinode));

  head->next = z;
  z->next = z;
  return head;
}

int free_silist(silist head)
{
  silist si = head->next;
  silist sinext;
  
  while((sinext = si->next) != si)
    {
      free(si);
      si = sinext;
    }
  return 0;
}

silist si_insafter(silist si,int i)
{
  silist newsi = (silist) malloc(sizeof(struct _sinode));
  
  newsi->key = i;
  newsi->next = si->next;
  si->next = newsi;
  return newsi;
}

silist si_delnext(silist si)
{
  si->next = si->next->next;
  free(si->next);
  return(si);
}

int si_len(silist si)
{
  int len = 0;
  si = si->next;
  while(si->next != si)
    {
      si = si->next;
      len++;
    }
  return len;
}

int si_dith(silist si,int dith)
{
  while(dith-- > 0)  /* dith == 1 is first */
      si = si->next;
  return si->key;
}

/* some special fast routines 
 */

silist si_fastcopy(silist si)
{
  silist si_act = si->next;
  silist newsi,new_act;

  newsi = malloc(sizeof(struct _sinode) * (si_len(si)+2)); 
  new_act = newsi;
  new_act->next = new_act+1;
  new_act++;
  si_act = si->next;
  while(si_act->next != si_act)
    {
      new_act->key = si_act->key;
      new_act->next = new_act + 1;
      new_act++;
      si_act = si_act->next;
    }
  new_act->next = new_act;
  return newsi;
}


